@@ -23,22 +23,28 @@
 
 #include "../mode_data/box_data_details.h"
 
+namespace Glom
+{
+
 class Box_Data_Details_Find : public Box_Data_Details
 {
 public: 
   Box_Data_Details_Find();
   virtual ~Box_Data_Details_Find();
 
+  virtual bool init_db_details(const Glib::ustring& table_name);
+
   virtual Gtk::Widget* get_default_button(); //override
-    
+
 protected:
 
-  virtual void fill_from_database(); //override.
+  virtual bool fill_from_database(); //override.
   virtual void fill_related(); //override.
 
-  virtual void on_adddel_user_changed(guint row, guint col); //override
-
+  virtual void on_flowtable_field_edited(const sharedptr<const LayoutItem_Field>& id, const Gnome::Gda::Value& value);
 
 };
 
+} //namespace Glom
+
 #endif
